from aud.AudFile import AudFile
import arrow
import os
from aud.AudLib import setupLogger

logger = setupLogger()

class AudDir:
    '''
    A wrapper for a directory path to ease use
    '''
    extensions = [".wav", ".mp3"]
    files = []
    output_directory = "_Processed"
    count = 0
    dir = ""

    def __init__(self, _dir, _output_directory="_Processed"):
        '''
        Initialization
        '''
        self.dir = _dir
        self.update()

    def __len__(self):
        return self.count

    def __getitem__(self, _index):
        return self.files[_index]

    def __contains__(self, value):
        ret = False
        if value in self.files:
            ret = True
        return ret

    def update(self):
        '''
        Update class to make sure changes are reflected
        '''
        self.files=[]
        _files = os.listdir(self.dir)
        for _file in _files:
            _file = AudFile(os.path.abspath(self.dir + '\\' + _file))
            if os.path.isfile(str(_file)):
                if str(_file.extension).lower() in str(self.extensions).lower():
                    self.count += 1
                    self.files.append(_file)

    ### UTILITIES ###
    def log(self, _dirpath):
        '''
        log the files that match the list of extensions into a meta file
        '''
        output = ""
        path = os.path.abspath(_dirpath)
        _files = os.listdir(path)
        ## Analyse each file
        for file in _files:
            filepath = os.path.abspath(file)
            if filepath.suffix in self.extensions:
                output += '' + str(path) + "\\" + str(filepath) + '\n'

        ## Summary of files
        output += '\n\nSUMMARY\n'
        output += "Count: " + str(self.count) + '\n'
        output += "Timestamp: " + str(arrow.now())
        self.writeFile(_dirpath, "meta", output)

    def writeFile(self, _dirpath, _name, _content, _filetype=".txt"):
        '''
        A quick helper to create a simple file cleanly
        '''
        file = open(_dirpath + "/" + _name + _filetype, "w")
        file.write(_content)
        file.close()

    ### SET METHODS ###
    def setOutputDir(self, _dirpath):
        '''
        Select an output directory
        '''
        if (Path(_dirpath).is_dir()):
            self.output_directory = _dirpath

    ### OS IO METHODS ###
    def renameUpper(self):
        '''
        Rename file to UPPERCASE
        '''
        for file in self.files:
            file.renameUpper()
        self.update()

    def renameLower(self):
        '''
        Rename file to lowercase
        '''
        for file in self.files:
            file.renameLower()
        self.update()

    def renameReplaceSpaces(self):
        '''
        Rename file to replace_all_spaces_with_underscored
        '''
        for file in self.files:
            file.renameReplaceSpaces()
        self.update()

    def renamePrepend(self, _prefix):
        for file in self.files:
            file.renameReplaceSpaces(_prefix)
        self.update()

    ### UNORGANIZED METHODS ###
    def convertTo(self, _extension=".wav", _target_samplerate=44100, _target_bitdepth=16, _target_bitrate="320k"):
        for file in self.files:
            file.convertTo(_extension, _target_samplerate, _target_bitdepth, _target_bitrate)

    def pad(self, _in=0.0, _out=0.0):
        for file in self.files:
            file.pad(_in, _out)

    def fade(self, _in=0.0, _out=0.0, _type='log|lin', _ratio=0.0):
        for file in self.files:
            file.fade(_in, _out, _type, _ratio)

    def move(self, _target_directory):
        for file in self.files:
            file.move(_target_directory)
        self.update()

    def metadata(self, tags):
        for file in self.files:
            file.metadata(tags)



import os
from aud.AudLib import setupLogger, createFolder
from pydub import AudioSegment

logger = setupLogger()

class AudFile:
    '''
    A wrapper for a file path to ease use
    '''
    filepath = ""
    name = ""
    extension = ""
    base = ""
    input_file = ""
    output_directory = "_Processed"

    def __init__(self, _input_file, _output_directory="_Processed"):
        try:
            path = os.path.abspath(_input_file)
            self.output_directory = _output_directory
            self.filepath = path
            self.name = os.path.basename(path)
            self.base = path.rstrip(self.name)
            self.extension = str('.' + self.name.split('.')[1].lower())
        except:
            del self

    ### override string and print
    def __repr__(self):
        return str(self.filepath)

    def __str__(self):
        return str(self.filepath)

    def __add__(self, _string):
        return str(self.filepath + _string)

    ### OS IO METHODS ###
    def renameUpper(self):
        '''
        Rename file to UPPERCASE
        '''
        logger.info("renameUpper:" + self.filepath)
        os.rename(self.filepath, self.base + '\\' + self.name.upper())

    def renameLower(self):
        '''
        Rename file to lowercase
        '''
        logger.info("renameLower:" + self.filepath)
        os.rename(self.filepath, self.base + '\\' + self.name.lower())

    def renameReplaceSpaces(self):
        '''
        Rename file to replace_all_spaces_with_underscored
        '''
        logger.info("renameReplaceSpaces:" + self.filepath)
        os.rename(self.filepath, self.base + '\\' + self.name.replace(" ", "_"))

    def renamePrepend(self, _prefix):
        '''
        Add a prefix for the filename
        '''
        logger.info("renamePrepend:" + self.filepath)
        os.rename(self.filepath, self.base + '\\' + _prefix + self.name)

    ### UNORGANIZED METHODS ###
    def convertTo(self, _extension=".wav", _target_samplerate=44100, _target_bitdepth=16, _target_bitrate="320k"):
        logger.info("convertTo:" + self.filepath)

        createFolder(self.base + self.output_directory)

        # Get FFMPEG to run the job
        ## THIS NEEDS TO BE REDOONE ##
        command = ""
        if _extension.lower() == ".mp3":
            ## full 320k with constant bitrate, may be wasteful
            if _target_bitrate.lower() == "320k":
                command = "ffmpeg -i {0} -codec:a libmp3lame -b:a 320k {1}.mp3".format(self.filepath, self.base + self.output_directory + "\\" + self.name.split(".")[0])
            ## variable bitrate
            elif _target_bitrate.lower() == "0":
                command = "ffmpeg -i {0} -codec:a libmp3lame -q:a 0 {1}.mp3".format(self.filepath, self.base + self.output_directory + "\\" + self.name.split(".")[0])
        elif _extension.lower() == ".wav":
            if _target_bitdepth == 16:
                command = "ffmpeg -i {0} {1}.wav".format(self.filepath, self.base + self.output_directory + "\\" + self.name.split(".")[0])
            elif _target_bitdepth == 24:
                command = "ffmpeg -i {0} -acodec s24le -ar {1} {2}.wav".format(self.filepath, _target_samplerate, self.base + self.output_directory + "\\" + self.name.split(".")[0])
            elif _target_bitdepth == 32:
                command = "ffmpeg -i {0} -acodec s32le -ar {1} {2}.wav".format(self.filepath, _target_samplerate, self.base + self.output_directory + "\\" + self.name.split(".")[0])
            elif _target_bitdepth == 64:
                command = "ffmpeg -i {0} -acodec s64le -ar {1} {2}.wav".format(self.filepath, _target_samplerate, self.base + self.output_directory + "\\" + self.name.split(".")[0])

        if command:
            logger.debug("COMMAND:" + command)
            os.system(command)
                #os.system("ffmpeg -i " + "\"" + str(self.filepath) + "\"" + " -vn -acodec pcm_s16le -ac 1 -ar " + str(_target_samplerate) + " -f " + _extension.lstrip(".") + " " + "\"" + self.base + self.output_directory + "\\" + self.name.split(".")[0] + _extension + "\"")

    def pad(self, _in=0.0, _out=0.0):
        '''
        Add leading and trailing blank audio to an audio file
        '''
        #logger.info("pad:" + self.filepath)

        # create processed folder
        createFolder(self.base + self.output_directory)

        leading_segment = AudioSegment.silent(duration=(1000*_in))
        trailing_segment = AudioSegment.silent(duration=(1000*_out))

        if self.extension ==".mp3":
            logger.info("Processing mp3: " + self.filepath)
            #read wav file to an audio segment
            audio = AudioSegment.from_mp3(self.filepath)

            # Add above two audio segments
            final_song = leading_segment + audio + trailing_segment

            #Either save modified audio
            final_song.export(self.base + '/' + self.output_directory + '/' + self.name, format="mp3")

        elif self.extension ==".wav":
            logger.info("Processing wav: " + self.filepath)
            #read wav file to an audio segment
            audio = AudioSegment.from_wav(self.filepath)

            # Add above two audio segments
            final_song = leading_segment + audio + trailing_segment

            #Either save modified audio
            final_song.export(self.base + '/' + self.output_directory + '/' + self.name, format="wav")

        elif self.extension ==".ogg":
            logger.info("Processing ogg: " + self.filepath)
            #read wav file to an audio segment
            audio = AudioSegment.from_ogg(self.filepath)

            # Add above two audio segments
            final_song = leading_segment + audio + trailing_segment

            #Either save modified audio
            final_song.export(self.base + '/' + self.output_directory + '/' + self.name, format="ogg")

        elif self.extension ==".flv":
            logger.info("Processing flv: " + self.filepath)
            #read wav file to an audio segment
            audio = AudioSegment.from_flv(self.filepath)

            # Add above two audio segments
            final_song = leading_segment + audio + trailing_segment

            #Either save modified audio
            final_song.export(self.base + '/' + self.output_directory + '/' + self.name, format="flv")

        ## This works with ogg or flv too

    def fade(self, _in=0.0, out=0.0, _type='log|lin', _ratio=0.0):
        '''
        Add fades to an audio file
        '''
        logger.info("fade:" + self.filepath)

    def move(self, _target_directory):
        '''
        Move file to another folder
        '''
        logger.info("moving:" + self.filepath)

    def metadata(self, _tags={}):
        '''
        add tags to a file like tags={"Artist": "Nobukazu Takemura", "Type": "Sfx"}
        You can add any tag you like but if you want them to come up in a particular
        application you will need to lookup what they use
        '''
        logger.info("metadata: " + self.filepath)

        if self.extension ==".mp3":
            audio = AudioSegment.from_mp3(self.filepath)
            audio.export(self.base + '/' + self.output_directory + '/' + self.name, format="mp3", tags=_tags)
        if self.extension ==".wav":
            audio = AudioSegment.from_wav(self.filepath)
            audio.export(self.base + '/' + self.output_directory + '/' + self.name, format="wav", tags=_tags)
        if self.extension ==".ogg":
            audio = AudioSegment.from_ogg(self.filepath)
            audio.export(self.base + '/' + self.output_directory + '/' + self.name, format="ogg", tags=_tags)
        if self.extension ==".flv":
            audio = AudioSegment.from_flv(self.filepath)
            audio.export(self.base + '/' + self.output_directory + '/' + self.name, format="flv", tags=_tags)

        #read wav file to an audio segment
        audio = AudioSegment.from_flv(self.filepath)

        # Add above two audio segments
        final_song = leading_segment + audio + trailing_segment

        #Either save modified audio
        final_song.export(self.base + '/' + self.output_directory + '/' + self.name, format="flv")



import aud

logger = setupLogger()

dirpath = aud.AudDir("test")

dirpath.extensions = [".wav", ".mp3"]

dirpath.setOutputDir("_Processed")

dirpath.metadata(["Artist": "Nobukazu Takemura", "Type": "sfx", "Keywords": "sword oneshot stereo"])

dirpath.renameUpper()

dirpath.renameLower()

dirpath.pad(_in=2.0, _out=3.5)

dirpath.log("_Processed")

dirpath.convertTo(_extension=".wav", _target_samplerate=44100, _target_bitdepth=16)

filepath = aud.AudDir("test.wav")

filepath.renameReplaceSpaces()

filepath.renamePrepend("sword_sfx_")

filepath.convertTo(_extension=".mp3", _target_samplerate=44100, _target_bitrate="320k") #bitrate="0" for lossy compression

logger.debug(filepath)
# C:\Folder\test.wav



"""
This is the documentation for aud
"""
#import .AudFile
#import .AudDir
#import .AudLib
from aud.AudFile import AudFile
from aud.AudDir import AudDir
from aud.AudLib import setupLogger, createFolder
import begin

@begin.start(auto_convert=True)
def main(file: 'File' = "", dir: 'Directory' = "", rename: 'Rename' = [], convert: 'Convert' = [], pad: 'Padding' = [], metadata: 'Metadata Tags' = [], extensions: 'Extensions' = []):
    '''
    aud: Quick tools for an audio studio environment
    '''
    logger = setupLogger()
    ## Decipher arguments and run the right command
    ## Designed to work well in the interpreter
    logger.debug("Import to start")

    ### SINGLE FILE COMMANDS ###
    if file != "":
        _file = AudFile(file)
        ## RENAME MODULES ##
        if rename[0].lower() == "lower":
            _file.renameLower()
        elif rename[0].lower() == "upper":
            _file.renameUpper()
        elif rename[0].lower() == "replacespaces":
            _file.renameReplaceSpaces()
        elif rename[0].lower() == "prepend":
            prefix = rename[1]
            _file.renamePrepend(prefix)

        ## METADTA MODULES ##
        if metadata != []:
            _file.metadata(metadata)

        ## FFMPEG MODULES ##
        ext = ".wav"
        sr = "44100"
        bd = "16"
        dr = "320k"
        if convert != []:
            if convert['extension']:
                ext = convert['extension']
            if convert['samplerate']:
                sr = convert['samplerate']
            if convert['bitdepth']:
                bd = convert['bitdepth']
            if convert['bitrate']:
                dr = convert['bitrate']
            _file.convertTo(_extension=ext, _target_samplerate=sr, _target_bitdepth=bd, _target_bitrate=br)

        _in = 0
        _out = 0
        if pad != []:
            if pad['in']:
                _in = pad['in']
            if pad['out']:
                _out = pad['out']
            _file.pad(_in=_in, _out=_out)

    ### DIR COMMANDS ###
    elif dir != "":
        _dir = AudDir(dir)
        # load extensions
        if extensions != []:
            _dir.extenstions = extensions

        ## RENAME MODULES ##
        if rename[0].lower() == "lower":
            _dir.renameLower()
        elif rename[0].lower() == "upper":
            _dir.renameUpper()
        elif rename[0].lower() == "replacespaces":
            _dir.renameReplaceSpaces()
        elif rename[0].lower() == "prepend":
            prefix = rename[1]
            _dir.renamePrepend(prefix)

        ## METADTA MODULES ##
        if metadata != []:
            _dir.metadata(metadata)

        ## FFMPEG MODULES ##
        ext = ".wav"
        sr = "44100"
        bd = "16"
        dr = "320k"
        if convert != []:
            if convert['extension']:
                ext = convert['extension']
            if convert['samplerate']:
                sr = convert['samplerate']
            if convert['bitdepth']:
                bd = convert['bitdepth']
            if convert['bitrate']:
                dr = convert['bitrate']
            _dir.convertTo(_extension=ext, _target_samplerate=sr, _target_bitdepth=bd, _target_bitrate=br)

        _in = 0.0
        _out = 0.0
        if pad != []:
            if pad['in']:
                _in = pad['in']
            if pad['out']:
                _out =pad['out']
            _dir.pad(_in=_in, _out=_out)



import colorlog
from os import makedirs
from os.path import exists

def setupLogger():
    '''
    Config and setup the logger
    '''
    ### SETUP LOGGER ###
    logger = colorlog.getLogger()
    # DEBUG INFO WARNING ERROR CRITICAL #
    logger.setLevel(colorlog.colorlog.logging.DEBUG)
    handler = colorlog.StreamHandler()
    handler.setFormatter(colorlog.ColoredFormatter())
    logger.addHandler(handler)
    return logger

def createFolder(path):
    '''
    Create folder if it does not already exist
    '''
    try:
        if not exists(path):
            makedirs(path)
    except OSError:
        logger.error("Creating directory failed")



import os
from setuptools import setup

import sys
if sys.version_info < (2,7):
    sys.exit('Sorry, Python < 2.7 is not supported')

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "aud",
    version = "0.1.12",
    author = "Zach Howard",
    author_email = "zach.d.howard@gmail.com",
    description = ("Quick tools for audio busy-work"),
    license = "MIT",
    keywords = "audio files tools easy",
    url = "http://github.com/zdhoward/aud",
    packages=['aud'],
    py_modules=['aud.AudDir', 'aud.AudFile', 'aud.AudLib', 'aud'],
    long_description=read('README.md'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=['pydub', 'arrow', 'colorlog', 'setuptools>24.2.0', 'pip>9.0.0']
)



from aud import *

#logger = setupLogger()


#dirpath = AudDir("test")

#dirpath.pad(2.0,2.0)
#dirpath.renameReplaceSpaces()
#dirpath.convertTo(".wav", _target_samplerate=44100)

dir = "test"
dirpath = aud.AudDir(dir)
logger = aud.setupLogger()
logger.debug("Performing a TEST on: {0}".format(dir))
